<?php

class ibackuper extends cmsFrontend {

    public function before($action_name) {

        if (!cmsUser::isAdmin()) {
            cmsCore::error404();
        }

        parent::before($action_name);

    }

    public function getCtype($id) {

        // получаем сам тип контента
        $ctype = $this->model_content->getContentType($id);
        if (!$ctype) {
            return [];
        }

        // получаем наборы
        $datasets = $this->model_content->getContentDatasets($id);

        // получаем категории
        $cats = $this->model_content->limit(0)->getCategoriesTree($ctype['name'], false);

        // получаем поля
        $fields = $this->model_content->getContentFields($ctype['name']);

        // очищаем поля от ненужных значений
        foreach ($fields as $name => $field) {
            unset($fields[$name]['handler_title'], $fields[$name]['handler']);
        }

        // получаем свойства
        $props = $this->model_content->limit(0)->getContentProps($ctype['name']);

        // получаем связи свойств с категориями
        $props_bind = $this->model->getPropsBinds($ctype['name']);

        // получаем модераторов типа контента
        $moderators = $this->model_moderation->getContentTypeModerators($ctype['name']);

        // получаем связи типов контента
        $relations = $this->model_backend_content->getContentRelations($id);

        if ($relations) {
            foreach ($relations as $id => $relation) {
                $relations[$id]['child_ctype'] = $this->model_content->getContentType($relation['child_ctype_id']);
            }
        }

        // получаем настройки доступов
        $rules = cmsPermissions::getRulesList('content');

        $perm_values = cmsPermissions::getPermissions($ctype['name']);

        // получаем настройки компонента генератора карты сайта
        $sitemap_options = cmsController::loadOptions('sitemap');

        // получаем настройки пресетов изображений
        $images_presets = $this->model_images->get('images_presets', function ($item) {
            unset($item['id']);
            $item['wm_image'] = cmsModel::yamlToArray($item['wm_image']);
            return $item;
        }, 'name');

        return [
            'ctype'           => $ctype,
            'cats'            => $cats,
            'fields'          => $fields,
            'props'           => $props,
            'props_bind'      => $props_bind,
            'datasets'        => $datasets,
            'moderators'      => $moderators,
            'relations'       => $relations,
            'rules'           => $rules,
            'perm_values'     => $perm_values,
            'sitemap_options' => !empty($sitemap_options['sources']['[content|' . $ctype['name'] . ']']) ? true : false,
            'images_presets'  => $images_presets
        ];

    }

    public function addCtypeFields($ctype_name, $fields) {

        $content_table_name =  $this->model_content->table_prefix . $ctype_name;
        $fields_table_name  =  $this->model_content->table_prefix . $ctype_name . '_fields';

        $sql = "TRUNCATE TABLE {#}{$fields_table_name}";
        $this->model_content->db->query($sql);

        foreach ($fields as $name => $field) {

            if (!in_array($name, array('title', 'user', 'date_pub')) && !$this->model_content->db->isFieldExists($content_table_name, $field['name'])) {

                $field_class  = 'field' . string_to_camel('_', $field['type']);
                $field_parser = new $field_class(null, (isset($field['options']) ? ['options' => $field['options']] : null));

                if (!$field_parser->is_virtual){

                    $sql = "ALTER TABLE {#}{$content_table_name} ADD `{$field['name']}` {$field_parser->getSQL()}";
                    $this->model_content->db->query($sql);

                    $field_parser->hookAfterAdd($content_table_name, $field, $this->model_content);

                    if ($field_parser->is_denormalization) {

                        $cfield_name = $field['name'] . cmsFormField::FIELD_CACHE_POSTFIX;
                        $sql = "ALTER TABLE {#}{$content_table_name} ADD `{$cfield_name}` {$field_parser->getCacheSQL()};";
                        $this->model_content->db->query($sql);

                    }

                    if (!empty($field['is_in_filter']) && $field_parser->allow_index) {
                        $this->model_content->db->addIndex($content_table_name, $field['name']);
                    }

                }

                cmsEventsManager::hook('ctype_field_after_add', array($field, $ctype_name, $this->model_content));

                // если есть опция полнотекстового поиска
                if (!$field_parser->is_virtual && is_array($field['options']) && !empty($field['options']['in_fulltext_search'])) {

                    // получаем полнотекстовый индекс для таблицы, он может быть только один
                    $fulltext_index = $this->model_content->db->getTableIndexes($content_table_name, 'FULLTEXT');

                    if ($fulltext_index) {

                        // название индекса
                        $index_name = key($fulltext_index);

                        // поля индекса
                        $index_fields = $fulltext_index[$index_name];

                        // ищем, нет ли такого поля уже в индексе, мало ли :-)
                        $key = array_search($field['name'], $index_fields);

                        // не нашли, добавляем
                        if ($key === false) {

                            // удаляем старый индекс
                            $this->model_content->db->dropIndex($content_table_name, $index_name);

                            // создаем новый
                            $this->model_content->createFullTextIndex($ctype_name, $field['name']);

                        }

                    } else {
                        $this->model_content->createFullTextIndex($ctype_name, $field['name']);
                    }

                }

            }

            $this->model_content->db->insert($fields_table_name, $field);

        }

        cmsCache::getInstance()->clean("content.fields.{$ctype_name}");

        return true;

    }

    public function getFormIforms($id) {

        $form = $this->model_iforms->getFormItemsById($id);

        if (!$form) {
            return false;
        }

        unset($form['frequency']);

        $fields = $this->model_iforms->getFormFields('iforms_fields', $form['id'], 'iforms_id');

        foreach ($fields as $name => $field) {
            unset($fields[$name]['handler_title'], $fields[$name]['handler']);
        }

        return [
            'form'   => $form,
            'fields' => $fields
        ];

    }

}
